classdef SCDclass_taskmdsloadprevADC < SCDclass_task
    % This is a special class which loads previous
    % values of the ADCs for the acquiring nodes
    properties
        modelbus
        node
        workspacedatabasestructure
    end
    
    properties(Access=private)
        logdecimate
    end
    
    methods
        
        function obj=SCDclass_taskmdsloadprevADC(id, varargin)   
            obj@SCDclass_task(id, varargin);
            obj.cparser.addRequired('modelbus',@(x) ischar(x));
            obj.cparser.addParameter('node', 0, @(x) (isnumeric(x) && ((x==1) || x==2 || x==7 || x==9)));
            obj=obj.parseconstructorcommon(id, varargin);
            obj.modelbus=obj.cparser.Results.modelbus;
            obj.node=obj.cparser.Results.node;
            switch(obj.node)
                case 1
                    obj.workspacedatabasestructure='SCDsimdata.SCDnode01simdata.adc';                    
                    obj.logdecimate=20;
                case 2
                    obj.workspacedatabasestructure='SCDsimdata.SCDnode02simdata.adc';
                    obj.logdecimate=20;
                case 7
                    obj.workspacedatabasestructure='SCDsimdata.SCDnode07simdata.adc';
                    obj.logdecimate=20;
                case 9
                    obj.workspacedatabasestructure='SCDsimdata.SCDnode09simdata.adc'; 
                    obj.logdecimate=100;
                otherwise
                    obj.workspacedatabasestructure='';
            end    
            obj.classname=mfilename;
        end
        
        function init(obj, shot)
            mdsconnect(obj.mdsserver);
            mdsopen(obj.mdstree, shot);
            
            try
                switch(obj.node)
                    case 1
                        obj.actualizeNode(shot, 96, false, 0);
                    case 2
                        obj.actualizeNode(shot, 192, false, 0);
                    case 7
                        obj.actualizeNode(shot, 96, true, 0);
                    case 9
                        obj.actualizeNode(shot, 768, false, 2);
                        obj.actualizeTimeandSample(shot, 1,2);
                end
            catch ME
                fprintf('\n');
                warning('SCDclass_taskmdsloadprevADC:init','Error initializing ADC bus for node %d.',obj.node);
                display(getReport(ME));
            end
         end
            
        function [obj, value] = getdata(obj,shot)
            %obj=obj.actualizegetcmd('mdsvalue(''%s'')', shot);            
            %value=eval(obj.getcommand);
            
            switch(obj.node)
                case 1
                      [~,value]=obj.getdataNode01ACQ196(shot);
                case 2
                    % Here we need to switch betweeen old and new acquisition unit 
                    % TODO: add tcv standard in case data is not available
                    if shot<65113
                      [~,value]=obj.getdataNode02ACQ196(shot); 
                    else
                      [~,value]=obj.getdataNode02ACQ2106(shot); 
                    end
                case 7
                      [~,value]=obj.getdataNode07(shot, 256);
                case 9
                      [~,value,~,~]=obj.getdataNode09(shot);                    
            end                
        end
        
        function printinfo(obj)
            obj.printinfocommon;
            fprintf('  Init model bus: %s, node: %d, ws base struct: %s\n',obj.modelbus, obj.node, obj.workspacedatabasestructure);
        end
        
            
        function entrystring = genMARTe2entry(obj, shot)
           % TODO: this is a workaround which works by chance, need a more accurate fix 
           entrystring = ''; 
        end
        
    end
    
    %methods(Access = private)
    methods
        
        function obj = actualizeTimeandSample(obj, shot, timeidx, sampleidx)
            dd=SCDconf_getdatadict(obj.datadictionary);
            adcbus=dd.getEntry(obj.modelbus).getValue;
            [~,~,timetrace,samplecnttrace]=obj.getdataNode09(shot);
            fprintf('Actualizing time and samplecnt for bus: ''%s'' (%s, shot %d)\n', obj.modelbus, obj.classname, shot);
            assignin('base','temp',timetrace);
            assigncmd=sprintf('%s.%s=temp;',obj.workspacedatabasestructure, adcbus.Elements(timeidx).Name);
            evalin('base',assigncmd);                
            assignin('base','temp',samplecnttrace);
            assigncmd=sprintf('%s.%s=temp;',obj.workspacedatabasestructure, adcbus.Elements(sampleidx).Name);
            evalin('base',assigncmd);                
            evalin('base','clear temp'); 
        end
        
        function obj = actualizeNode(obj, shot, nadcs, packetized, startoffset)
            dd=SCDconf_getdatadict(obj.datadictionary);
            adcbus=dd.getEntry(obj.modelbus).getValue;
            [~,data]=obj.getdata(shot);
            T=timeseries; 
            if obj.verbose==1
               fprintf('Actualizing data for bus: ''%s'' (%s, shot %d) 0%% ', obj.modelbus, obj.classname, shot);
            end
            for ii=1:nadcs
                if ~packetized
                 T.Time=data.Time;
                 T.Data=int16(data.Data(:,ii));
                else
                 T=data.get(['adc' num2str(ii)]);
                end    
                assignin('base','temp',T);
                assigncmd=sprintf('%s.%s=temp;',obj.workspacedatabasestructure, adcbus.Elements(ii+startoffset).Name);
                evalin('base',assigncmd);                
                if obj.verbose==1 && mod(ii,obj.logdecimate)==0
                    fprintf('%.0f%% ', ii/nadcs*100);
                end
            end
            evalin('base','clear temp');
            if obj.verbose==1
                fprintf('100%% \n');
            end   
        end
                           
%% Node 01 loading functions

        function [obj, value] = getdataNode01ACQ196(obj, shot)
            T=timeseries;
            timebase=mdsvalue('dim_of(\top.crpprt01.board1.adc_01)'); %% TODO: check with marte
            d_time = double(mdsvalueraw( '\top.crpprt01.params:d_time' ))*1.0e-6;
            %timebase = round(timebase.*1/d_time)*d_time-d_time/2;
            timebase = round(timebase.*1/d_time)*d_time-d_time/100;
            T.Time=timebase;
            T.Data=zeros(numel(timebase),96);
            datamatrix=zeros(numel(timebase),96);
            if obj.verbose==1
               fprintf('Getting data for bus: ''%s'' (%s, shot %d) 0%% ', obj.modelbus, obj.classname, shot);
            end
            for ii=1:96
                % Node 01 data are loaded in the stored order since
                % up to now there is only old node 01

                channelstr=sprintf('\\top.crpprt01.board1.adc_%02d.raw',physical2logical(ii));
                datamatrix(:,ii)=int16(mdsvalue(channelstr));   
                if obj.verbose==1 && mod(ii,obj.logdecimate)==0 
                    fprintf('%.0f%% ', ii/96*100);
                end
            end
            T.Data=datamatrix; % this speeds up a lot the loading
            value=T;
            if obj.verbose==1
                fprintf('100%% \n');
            end  
        end

%% Node 02 loading functions

        function [obj, value] = getdataNode02ACQ196(obj, shot)
            T=timeseries;
            timebase=mdsvalue('dim_of(\top.crpprt02.board1.adc_01)'); 
            d_time = double(mdsvalueraw( '\top.crpprt02.params:d_time' ))*1.0e-6;
            %timebase = round(timebase.*1/d_time)*d_time-d_time/2;
            timebase = round(timebase.*1/d_time)*d_time-d_time/100;
            T.Time=timebase;
            T.Data=zeros(numel(timebase),192);
            datamatrix=zeros(numel(timebase),192);
            if obj.verbose==1
               fprintf('Getting data for bus: ''%s'' (%s, shot %d) 0%% ', obj.modelbus, obj.classname, shot);
            end
            for ii=1:96
                % NOTE: physical2logical remapping is only for
                % data acquired with the ACQ196 system, not for the new
                % 2106 based one
                channelstr=sprintf('\\top.crpprt02.board1.adc_%02d.raw',physical2logical(ii));
                datamatrix(:,ii)=int16(mdsvalue(channelstr));   
                if obj.verbose==1 && mod(ii,obj.logdecimate)==0 
                    fprintf('%.0f%% ', ii/192*100);
                end
            end
            for ii=1:96
                channelstr=sprintf('\\top.crpprt02.board2.adc_%02d.raw',physical2logical(ii));
                datamatrix(:,ii+96)=int16(mdsvalue(channelstr));
                if obj.verbose==1 && mod(ii,obj.logdecimate)==0
                    fprintf('%.0f%% ', (ii+96)/192*100);
                end
            end
            T.Data=datamatrix; % this speeds up a lot the loading
            value=T;
            if obj.verbose==1
                fprintf('100%%\n');
            end  
        end
        
        function [obj, value] = getdataNode02ACQ2106(obj, shot)
            if shot>65826
                mdsnodename='node02';
            else
                mdsnodename='tcvrt02';
            end
            mdsopen('rtc',shot);
            T=timeseries;
            timebase=mdsvalue(['dim_of(\top.' mdsnodename '.adc.adc_001)']);
            d_time = double(mdsvalueraw(['\top.' mdsnodename '.params:d_time']))*1.0e-6;
            %timebase = round(timebase.*1/d_time)*d_time-d_time/2;
            timebase = round(timebase.*1/d_time)*d_time-d_time/100;
            timebase = timebase + d_time; % this fits perfectly sim/rt data on 68072
            T.Time=timebase;
            T.Data=zeros(numel(timebase),192);
            datamatrix=zeros(numel(timebase),192);
            if obj.verbose==1
               fprintf('Getting data for bus: ''%s'' (%s, shot %d) 0%% ', obj.modelbus, obj.classname, shot);
            end
            for ii=1:192
                channelstr=sprintf(['\\top.' mdsnodename '.adc.adc_%03d.raw'],ii);
                %T.Data(:,ii)=int16(mdsvalue(channelstr));  
                %T.Data(:,ii)=int16(mdsvalueraw(channelstr));                  
                datamatrix(:,ii)=int16(mdsvalueraw(channelstr));
                if obj.verbose==1 && mod(ii,obj.logdecimate)==0 
                    fprintf('%.0f%% ', ii/192*100);
                end
            end
            T.Data=datamatrix; % this speeds up a lot the loading
            value=T;
            if obj.verbose==1
                fprintf('100%% \n');
            end              
        end

%% Node 07 loading functions
        
        function [obj, value] = getdataNode07(obj, shot, samppck)
            nch=96;
            if shot>=70145 % first shot with new MARTe2 based node 07
                oldway=false;
            else
                oldway=true;
            end
            mdsopen('rtc',shot);
            if oldway
                timebase=mdsvalue(['dim_of(\top.crpprt07.board1.adc_01)']);
                d_time = double(mdsvalueraw(['\top.crpprt07.params:d_time']))*1.0e-6;
            else
                timebase=mdsvalue(['dim_of(\top.node07.adc.adc_001)']);
                d_time = double(mdsvalueraw(['\top.node07.params:d_time']))*1.0e-6;
            end
            timebase = timebase(1:samppck:end); % TODO: is this safe against lost cycles ?
            %timebase = round(timebase.*1/d_time)*d_time-d_time/2;
            timebase = round(timebase.*1/d_time)*d_time-d_time/100;
            %timebase = timebase + d_time; % this fits perfectly sim/rt data on 68072
            value = tscollection(timebase);
            if obj.verbose==1
               fprintf('Getting data for bus: ''%s'' (%s, shot %d) 0%% ', obj.modelbus, obj.classname, shot);
            end
            for ii=1:nch
                T=timeseries;             
                T.Time=timebase;
                if oldway
                    channelstr=sprintf(['\\top.crpprt07.board1.adc_%02d.raw'],ii);
                else
                    channelstr=sprintf(['\\top.node07.adc.adc_%03d.raw'],ii);                    
                end
                data=mdsvalueraw(channelstr);
                if isnumeric(data)
                  datamatrix=reshape(int16(data),[samppck,numel(timebase)])';
                  %datamatrix=reshape(int16(data),[samppck,numel(timebase)]);                
                else
                  datamatrix=int16(zeros(numel(timebase),samppck));
                  %datamatrix=int16(zeros(samppck,numel(timebase)));
                end
                T.Data=datamatrix;
                
                if obj.verbose==1 && mod(ii,obj.logdecimate)==0 
                    fprintf('%.0f%% ', ii/nch*100);
                end
                T.Name=['adc' num2str(ii)];
                value=addts(value,T);
            end
            %T.Data=datamatrix; % this speeds up a lot the loading
            %value.addts;
            if obj.verbose==1
                fprintf('100%% \n');
            end                          
        end    

%% Node 09 loading functions

        function [obj, value, time, samplecnt] = getdataNode09(obj, shot)
            mdsopen('rtc',shot);
            T=timeseries;
            T.TimeInfo.Units='samples';
            if shot>73708
                samplecnt=mdsvalue('node09.thread02.mems.mem_002');
                thomsontime=mdsvalue('node09.thread02.mems.mem_001');
                thread02time=double(mdsvalue('node09.thread02.stats.stat_002'));
            else
                samplecnt=mdsvalue('node09.thread02.mems.mem_005');
                thomsontime=mdsvalue('node09.thread02.mems.mem_006'); 
                thread02time=double(mdsvalue('node09.thread02.stats.stat_002'));
            end
            if(samplecnt(1)~=0)
                warning('SCDclass_taskmdsloadprevADC:init','node 09 thread 2 skipped the first cycle, check initial samples time carefully');
            end
            %timebase(samplecnt(1)+1:end)=timebase(samplecnt(1)+1:end)+thomsontime(1);
            timebase=1:numel(samplecnt);
            T.Time=timebase;
            T.Data=zeros(numel(timebase),768);
            datamatrix=zeros(numel(timebase),768);
            if obj.verbose==1
               fprintf('Getting data for bus: ''%s'' (%s, shot %d) 0%% ', obj.modelbus, obj.classname, shot);
            end
            for ii=1:768
                channelstr=sprintf(['node09.adc.adc_%03d.raw'],ii);
                adcthread01=int16(mdsvalueraw(channelstr));
                datamatrix(:,ii)=adcthread01(samplecnt+1);
                if obj.verbose==1 && mod(ii,obj.logdecimate)==0 
                    fprintf('%.0f%% ', ii/768*100);
                end
            end
            T.Data=datamatrix; % this speeds up a lot the loading
            value=T;
            if obj.verbose==1
                fprintf('100%% \n');
            end
            T.Data=single(thread02time');
            time=T;
            T.Data=int32(samplecnt');
            samplecnt=T;
        end

    end
end

